//	yf NetDiff dialog
//		2005.09.28 - 2005.11.25 Yoshiaki.Ueda
#include "wx/wx.h"
#include "yfNetDifferencer.h"
#include "yfNetConverter.h"
#include "yfNetMode.h"
#include <wx/dnd.h>
#include <wx/config.h>
#include <wx/filename.h>


static const wxString SYSTEM_NAME = wxT("ORIGINAL v0.20 (2005/11/25) (c)2005 ysfactory    Modified Version 0.21.01 (2023/12/28) SUIGYODO");
static const wxString FILE_DIALOG_WILDCARD =
	wxT("Net list (*.net *.lst *.txt *.ccf *.ecf)|*.net;*.lst;*.txt;*.ccf;*.ecf|All files (*.*)|*.*");

enum{
	IDC_FILE_1_NAME		=wxID_HIGHEST+1,
	IDC_FILE_1_MODE,
	IDC_FILE_1_SELECT,
	IDC_FILE_1_CONVERT,
	IDC_FILE_2_MODE,
	IDC_FILE_2_NAME,
	IDC_FILE_2_SELECT,
	IDC_FILE_2_CONVERT,
	IDC_EXECUTE,
	IDC_FILE_SAVE,
	IDC_DIFFERENCE
};

class yfNetDiffDlg : public wxDialog
{
	friend class DropTarget;
public:
	wxTextCtrl	m_file1Name;
	wxTextCtrl	m_file2Name;
	wxChoice	m_file1Mode;
	wxChoice	m_file2Mode;
	wxTextCtrl	m_difference;
	wxString	m_outFileName;
	yfNetDiffDlg();
protected:
	wxSizer*	m_file1Sizer;
	wxSizer*	m_file2Sizer;
	DropTarget*	m_dropTarget;
	wxConfig	m_config;
	void InitMode(wxChoice& comboBox);
private:
	void OnFile1SelectButtonClicked(wxCommandEvent& event);
	void OnFile2SelectButtonClicked(wxCommandEvent& event);
	void OnFile1ConvertButtonClicked(wxCommandEvent& event);
	void OnFile2ConvertButtonClicked(wxCommandEvent& event);
	void OnExecuteButtonClicked(wxCommandEvent& event);
	void OnFileSaveButtonClicked(wxCommandEvent& event);
	void OnOK(wxCommandEvent& event);
	DECLARE_EVENT_TABLE()
};

//MyDropTargetClass//==========================================================

class DropTarget : public wxFileDropTarget
{
	bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
	wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def);
public:
	DropTarget(yfNetDiffDlg& dialog);
	virtual ~DropTarget()
	{
	};
protected:
	yfNetDiffDlg& m_dialog;
};

DropTarget::DropTarget(yfNetDiffDlg& dialog)
	: m_dialog(dialog)
{
}

bool DropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
{
	wxRect rect1(m_dialog.m_file1Sizer->GetPosition(),
				m_dialog.m_file1Sizer->GetSize());
	if(rect1.Contains(x, y)){
		m_dialog.m_file1Name.SetValue(filenames[0]);
		return true;
	}
	wxRect rect2(m_dialog.m_file2Sizer->GetPosition(),
				m_dialog.m_file2Sizer->GetSize());
	if(rect2.Contains(x, y)){
		m_dialog.m_file2Name.SetValue(filenames[0]);
		return true;
	}
	return false;
}

wxDragResult DropTarget::OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
{
	wxRect rect1(m_dialog.m_file1Sizer->GetPosition(),
				m_dialog.m_file1Sizer->GetSize());
	if(rect1.Contains(x, y)) return wxDragCopy;
	wxRect rect2(m_dialog.m_file2Sizer->GetPosition(),
				m_dialog.m_file2Sizer->GetSize());
	if(rect2.Contains(x, y)) return wxDragCopy;
	return wxDragNone;
}

//yfNetDiffDlg//===============================================================

yfNetDiffDlg::yfNetDiffDlg()
	: wxDialog(NULL, -1, wxT("yfNetDiff"), wxDefaultPosition, wxDefaultSize,
		wxDEFAULT_DIALOG_STYLE | wxMINIMIZE_BOX | wxMAXIMIZE_BOX | wxRESIZE_BORDER)
	, m_outFileName(wxT("difference")), m_dropTarget(NULL)
	, m_config(wxT("yfNetDiff"), wxT("ysfactory"))
{

	wxBoxSizer* topsizer = new wxBoxSizer(wxVERTICAL);
	topsizer->SetMinSize(500, 300);

	wxBoxSizer* boxsizer;
	//File 1
	topsizer->Add(
		new wxStaticText(this, -1, wxT("File 1")),
		0,wxLEFT|wxTOP|wxRIGHT,5);

	m_file1Sizer = boxsizer = new wxBoxSizer(wxHORIZONTAL);
	m_file1Name.Create(this, IDC_FILE_1_NAME);
	boxsizer->Add(&m_file1Name,
		1,wxALIGN_CENTRE_VERTICAL|wxLEFT|wxRIGHT,5);

	m_file1Mode.Create(this, IDC_FILE_1_MODE);
	yfNetMode::InitChoice(m_file1Mode);
	boxsizer->Add(&m_file1Mode,
		0,wxALIGN_CENTRE_VERTICAL|wxLEFT|wxRIGHT,5);

	boxsizer->Add(
		new wxButton(this, IDC_FILE_1_SELECT, wxT("Select")),
		0,wxALIGN_CENTRE_VERTICAL|wxLEFT|wxRIGHT,5);

	boxsizer->Add(
		new wxButton(this, IDC_FILE_1_CONVERT, wxT("Convert")),
		0,wxALIGN_CENTRE_VERTICAL|wxLEFT|wxRIGHT,5);
	topsizer->Add(boxsizer,0,wxEXPAND);
	
	//File 2
	topsizer->Add(
		new wxStaticText(this, -1, wxT("File 2")),
		0,wxLEFT|wxTOP|wxRIGHT,5);

	m_file2Sizer = boxsizer = new wxBoxSizer(wxHORIZONTAL);
	m_file2Name.Create(this, IDC_FILE_2_NAME);
	boxsizer->Add(&m_file2Name,
		1,wxALIGN_CENTRE_VERTICAL|wxLEFT|wxRIGHT,5);

	m_file2Mode.Create(this, IDC_FILE_2_MODE);
	yfNetMode::InitChoice(m_file2Mode);
	boxsizer->Add(&m_file2Mode,
		0,wxALIGN_CENTRE_VERTICAL|wxLEFT|wxRIGHT,5);

	boxsizer->Add(
		new wxButton(this, IDC_FILE_2_SELECT, wxT("Select")),
		0,wxALIGN_CENTRE_VERTICAL|wxLEFT|wxRIGHT,5);

	boxsizer->Add(
		new wxButton(this, IDC_FILE_2_CONVERT, wxT("Convert")),
		0,wxALIGN_CENTRE_VERTICAL|wxLEFT|wxRIGHT,5);
	topsizer->Add(boxsizer,0,wxEXPAND);

	//Execute
	boxsizer = new wxStaticBoxSizer(
		new wxStaticBox(this, -1, wxT("")), wxHORIZONTAL);
	boxsizer->Add(new wxBoxSizer(wxHORIZONTAL),3);

	//??//wxButton* exeButton = new wxButton(this, IDC_EXECUTE, wxT("Execute"));
	//??//wxButton* saveButton = new wxButton(this, IDC_FILE_SAVE, wxT("File save"));
	boxsizer->AddStretchSpacer();

	boxsizer->Add(
		new wxButton(this, IDC_EXECUTE, wxT("Execute")),
		0,wxALL,5);
		
	boxsizer->Add(
		new wxButton(this, IDC_FILE_SAVE, wxT("File save")),
		0,wxALL,5);

	topsizer->Add(boxsizer,0,wxEXPAND|wxALL,5);
			
	//Difference	
	topsizer->Add(
		new wxStaticText(this, -1, wxT("Results")),
		0,wxLEFT|wxTOP|wxRIGHT,5);
	
	m_difference.Create(this, IDC_DIFFERENCE, "",
		wxDefaultPosition, wxDefaultSize, wxTE_RICH|wxTE_READONLY|wxTE_MULTILINE);
	topsizer->Add(
		&m_difference,
		1,wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM,5);

	//OK
	boxsizer = new wxBoxSizer(wxHORIZONTAL);
	boxsizer->Add(
		new wxStaticText(this, -1, SYSTEM_NAME),
		1,wxALIGN_CENTRE_VERTICAL|wxALL,5 );
	boxsizer->Add(new wxBoxSizer(wxHORIZONTAL),1);

	boxsizer->Add(
		new wxButton(this, wxID_OK, wxT("Exit")),
		0,wxALIGN_CENTRE_VERTICAL|wxALL,5);

	//boxsizer->Add(
	//	new wxButton(this, wxID_CANCEL, wxT("Cancel")),
	//	0,wxALIGN_CENTRE_VERTICAL|wxALL,5);
	topsizer->Add(boxsizer,0,wxEXPAND);
			
	SetSizer(topsizer);
	topsizer->SetSizeHints(this);
	
	//Drop
	m_dropTarget = new DropTarget(*this);
	SetDropTarget(m_dropTarget);
	
	//Load configuration
	wxRect rect = GetRect();
	rect.x = m_config.Read( wxT("WindowX"), rect.x );
	rect.x = m_config.Read( wxT("WindowY"), rect.y );
	rect.width = m_config.Read( wxT("WindowWidth"), rect.width );
	rect.height = m_config.Read( wxT("WindowHeight"), rect.height );
	SetSize(rect);
	m_file1Name.SetValue(
		m_config.Read( wxT("File1Name"), m_file1Name.GetValue() ) );
	m_file1Mode.SetStringSelection(
		m_config.Read( wxT("File1Mode"), m_file1Mode.GetStringSelection() ) );
	m_file2Name.SetValue(
		m_config.Read( wxT("File2Name"), m_file2Name.GetValue() ) );
	m_file2Mode.SetStringSelection(
		m_config.Read( wxT("File2Mode"), m_file2Mode.GetStringSelection() ) );
	//??//m_outFileName = m_config.Read( wxT("OutFileName"), m_outFileName );
};



void yfNetDiffDlg::OnOK(wxCommandEvent& event)
{
	//Save configuration
	wxRect rect = GetRect();
	m_config.Write( wxT("WindowX"), rect.x );
	m_config.Write( wxT("WindowY"), rect.y );
	m_config.Write( wxT("WindowWidth"), rect.width );
	m_config.Write( wxT("WindowHeight"), rect.height );
	m_config.Write( wxT("File1Name"), m_file1Name.GetValue() );
	m_config.Write( wxT("File1Mode"), m_file1Mode.GetStringSelection() );
	m_config.Write( wxT("File2Name"), m_file2Name.GetValue() );
	m_config.Write( wxT("File2Mode"), m_file2Mode.GetStringSelection() );
	m_config.Write( wxT("OutFileName"), m_outFileName );
	event.Skip();
}

BEGIN_EVENT_TABLE(yfNetDiffDlg, wxDialog)
	EVT_BUTTON(IDC_FILE_1_SELECT, yfNetDiffDlg::OnFile1SelectButtonClicked)
	EVT_BUTTON(IDC_FILE_2_SELECT, yfNetDiffDlg::OnFile2SelectButtonClicked)
	EVT_BUTTON(IDC_FILE_1_CONVERT, yfNetDiffDlg::OnFile1ConvertButtonClicked)
	EVT_BUTTON(IDC_FILE_2_CONVERT, yfNetDiffDlg::OnFile2ConvertButtonClicked)
	EVT_BUTTON(IDC_EXECUTE, yfNetDiffDlg::OnExecuteButtonClicked)
	EVT_BUTTON(IDC_FILE_SAVE, yfNetDiffDlg::OnFileSaveButtonClicked)
	EVT_BUTTON(wxID_OK, yfNetDiffDlg::OnOK)
END_EVENT_TABLE()

void yfNetDiffDlg::OnFile1SelectButtonClicked(wxCommandEvent& event)
{
	wxFileDialog dlg(this,
		wxT("Select - file 1"), wxT(""), m_file1Name.GetValue(),
		FILE_DIALOG_WILDCARD,
		wxFD_OPEN
		//|wxHIDE_READONLY
		);
	if( dlg.ShowModal()==wxID_OK ){
		m_file1Name.SetValue(dlg.GetPath());
	}
}

void yfNetDiffDlg::OnFile2SelectButtonClicked(wxCommandEvent& event)
{
	wxFileDialog dlg(this,
		wxT("Select - file 2"), wxT(""), m_file1Name.GetValue(),
		FILE_DIALOG_WILDCARD,
		wxFD_OPEN
		//|wxHIDE_READONLY
		);
	if( dlg.ShowModal()==wxID_OK ){
		m_file2Name.SetValue(dlg.GetPath());
	}
}

void yfNetDiffDlg::OnFile1ConvertButtonClicked(wxCommandEvent& event)
{
	if( m_file1Name.GetValue().IsEmpty() ){
		wxMessageBox(wxT("Please select the file."));
		return;
	}
	wxFileName outFileName(m_file1Name.GetValue());
	//Make filename
	outFileName.SetExt(wxT(""));
	//Select
	wxFileDialog dlg(this,
		wxT("Transfer - file 1"), wxT(""), wxT("convert"),	//??//outFileName.GetFullPath().GetData(),
		yfNetMode::GetDialogWildcard().GetData(),
		wxFD_SAVE|wxFD_OVERWRITE_PROMPT );
	if( dlg.ShowModal()==wxID_OK ){
		//Execute
		yfNetConverter::Execute(
			m_file1Name.GetValue(), yfNetMode::GetMode(m_file1Mode.GetSelection()),
			dlg.GetPath(), yfNetMode::GetMode(dlg.GetFilterIndex()) );
	}
}

void yfNetDiffDlg::OnFile2ConvertButtonClicked(wxCommandEvent& event)
{
	if( m_file2Name.GetValue().IsEmpty() ){
		wxMessageBox(wxT("Please select the file."));
		return;
	}
	wxFileName outFileName(m_file2Name.GetValue());
	//Make filename
	outFileName.SetExt(wxT(""));
	//Select
	wxFileDialog dlg(this,
		wxT("Convert - file 2"), wxT(""), wxT("convert"),	//??//outFileName.GetFullPath().GetData(),
		yfNetMode::GetDialogWildcard().GetData(),
		wxFD_SAVE|wxFD_OVERWRITE_PROMPT );
	if( dlg.ShowModal()==wxID_OK ){
		//Execute
		yfNetConverter::Execute(
			m_file2Name.GetValue(), yfNetMode::GetMode(m_file2Mode.GetSelection()),
			dlg.GetPath(), yfNetMode::GetMode(dlg.GetFilterIndex()) );
	}
}

void yfNetDiffDlg::OnExecuteButtonClicked(wxCommandEvent& event)
{
	if( m_file1Name.GetValue().IsEmpty() || m_file2Name.GetValue().IsEmpty() ){
		wxMessageBox(wxT("Please select the file."));
		return;
	}
	yfNetDifferencer diff;
	diff.Execute(
		m_file1Name.GetValue(), yfNetMode::GetMode(m_file1Mode.GetSelection()),
		m_file2Name.GetValue(), yfNetMode::GetMode(m_file2Mode.GetSelection()) );
	m_difference.Clear();
	diff.GetText(m_difference);
}

void yfNetDiffDlg::OnFileSaveButtonClicked(wxCommandEvent& event)
{
	wxFileDialog dlg(this,
		wxT("File save - difference"), wxT(""), m_outFileName,
		wxT("Text file (*.txt)|*.txt|All files (*.*)|*.*"),
		wxFD_SAVE|wxFD_OVERWRITE_PROMPT );
	if( dlg.ShowModal()==wxID_OK ){
		m_difference.SaveFile(m_outFileName=dlg.GetPath());
	}
}

//yfNetDiffApp//===============================================================

class yfNetDiffApp : public wxApp
{
	bool OnInit()
	{
		yfNetDiffDlg dlg;
		dlg.ShowModal();
		return false;
	};
};

IMPLEMENT_APP(yfNetDiffApp)
